/*
 * $Id: man.c,v 1.3 2002/05/06 09:59:43 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Handle player's walls.
 *
 * @version $Revision: 1.3 $
 */
#include <sys/bios.h>
#include <stdlib.h>

#include "putipati.h"
#include "man.h"
#include "dispmng.h"
#include "gamemng.h"
#include "stagemng.h"
#include "attrcmng.h"

Man man[2];

void initMen() {
  man[0].pos.x = (SCREEN_HEIGHT/2-16-4)<<8; man[0].pos.y = (SCREEN_WIDTH<<7)+MAN_LENGTH/2;
  man[0].pPos = man[0].pos;
  man[0].vel.x = man[0].vel.y = 0;
  man[1].pos.x = (SCREEN_HEIGHT/2+16-4)<<8; man[1].pos.y = (SCREEN_WIDTH<<7)+MAN_LENGTH/2;
  man[1].pPos = man[1].pos;
  man[1].vel.x = man[1].vel.y = 0;
  timerBlink = 1;
}

void clearMen() {
  int i;
  for ( i=0 ; i<12 ; i++ ) {
    sprite_set_location(MAN_SPRITE_IDX+i, DISAP_POS, DISAP_POS);
  }
  man[0].pos.x = man[0].pos.y = man[1].pos.x = man[1].pos.y = -99999l;
}

static int checkManWallHitV(int cx, int cy) {
  if ( cx<0 || cx>=STAGE_WIDTH || cy<0 || cy>=STAGE_HEIGHT-1 ) return 1;
  return block[cx][cy] | block[cx][cy+1];
}

static int checkManWallHitH(int cx, int cy) {
  if ( cx<0 || cx>=STAGE_WIDTH-1 || cy<0 || cy>=STAGE_HEIGHT ) return 1;
  return block[cx][cy] | block[cx+1][cy];
}

#define MAN_ACCEL 180
static int pKey = 0;
static int mhCnt = 0;

void moveMen(int key) {
  int i;
  Man *mn;
  int sx, sy;
  int ax, ay;
  int cx, cy;
  ax = ay = 0;
  if ( key & 0x0100 ) ax = -MAN_ACCEL;
  if ( key & 0x0200 ) ay =  MAN_ACCEL;
  if ( key & 0x0400 ) ax =  MAN_ACCEL;
  if ( key & 0x0800 ) ay = -MAN_ACCEL;
  if ( (pKey&0x0f00) == 0 ) {
    ax <<= 3; ay <<= 3;
  }
  man[0].vel.x += ax; man[0].vel.y += ay;
  ax = ay = 0;
  if ( key & 0x0010 ) ax = -MAN_ACCEL;
  if ( key & 0x0020 ) ay =  MAN_ACCEL;
  if ( key & 0x0040 ) ax =  MAN_ACCEL;
  if ( key & 0x0080 ) ay = -MAN_ACCEL;
  if ( (pKey&0x00f0) == 0 ) {
    ax <<= 3; ay <<= 3;
  }
  man[1].vel.x += ax; man[1].vel.y += ay;
  pKey = key;

  if ( abs(man[0].pos.x-man[1].pos.x) < MAN_WIDTH && 
       abs(man[0].pos.y-man[1].pos.y) < MAN_LENGTH ) {
    Vector tVel;
    man[0].pos = man[0].pPos; man[1].pos = man[1].pPos;
    tVel = man[0].vel; man[0].vel = man[1].vel; man[1].vel = tVel;
    mhCnt++; if ( mhCnt > 16 ) initMen();
  } else {
    mhCnt = 0;
  }
  for ( i=0 ; i<2 ; i++ ) {
    mn = &(man[i]);

    cx = (mn->pos.x+mn->vel.x)>>11; cy = mn->pos.y>>11; 
    switch ( i ) {
    case 0:
      if ( checkManWallHitV(cx-1, cy-4) || checkManWallHitV(cx+1, cy-4) ||
	   checkManWallHitH(cx, cy-1) || checkManWallHitH(cx, cy-6) ) {
	man[i].vel.x = -man[i].vel.x;
	man[i].pos.x = man[i].pPos.x;
      }
      break;
    case 1:
      if ( checkManWallHitV(cx+2, cy-4) || checkManWallHitV(cx, cy-4) ||
	   checkManWallHitH(cx, cy-1) || checkManWallHitH(cx, cy-6) ) {
	man[i].vel.x = -man[i].vel.x;
	man[i].pos.x = man[i].pPos.x;
      }
      break;
    }
    cx = mn->pos.x>>11; cy = (mn->pos.y+mn->vel.y)>>11; 
    switch ( i ) {
    case 0:
      if ( checkManWallHitV(cx-1, cy-4) || checkManWallHitV(cx+1, cy-4) ||
	   checkManWallHitH(cx, cy-1) || checkManWallHitH(cx, cy-6) ) {
	man[i].vel.y = -man[i].vel.y;
	man[i].pos.y = man[i].pPos.y;
      }
      break;
    case 1:
      if ( checkManWallHitV(cx+2, cy-4) || checkManWallHitV(cx, cy-4) ||
	   checkManWallHitH(cx, cy-1) || checkManWallHitH(cx, cy-6) ) {
	man[i].vel.y = -man[i].vel.y;
	man[i].pos.y = man[i].pPos.y;
      }
      break;
    }

    mn->pPos = mn->pos;
    vctAdd(&(mn->pos), &(mn->vel));
    mn->vel.x -= (mn->vel.x>>2); mn->vel.y -= (mn->vel.y>>2); 
    if ( vctSize(&mn->vel) > 1024 ) {
      mn->vel.x -= (mn->vel.x>>1);
      mn->vel.y -= (mn->vel.y>>1);
    }
    sx = mn->pos.y>>8; sy = mn->pos.x>>8;
    sprite_set_location(i*6+MAN_SPRITE_IDX, sx-8, sy);
    sprite_set_location(i*6+1+MAN_SPRITE_IDX, sx-16, sy);
    sprite_set_location(i*6+2+MAN_SPRITE_IDX, sx-24, sy);
    sprite_set_location(i*6+3+MAN_SPRITE_IDX, sx-32, sy);
    sprite_set_location(i*6+4+MAN_SPRITE_IDX, sx-40, sy);
    if ( timerBlink ) {
      switch ( i ) {
      case 0:
	sprite_set_location(i*6+5+MAN_SPRITE_IDX, sx-24, sy-8);
	break;
      case 1:
	sprite_set_location(i*6+5+MAN_SPRITE_IDX, sx-24, sy+8);
	break;
      }
    } else {
      sprite_set_location(i*6+5+MAN_SPRITE_IDX, DISAP_POS, DISAP_POS);
    }
  }
}
